package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.constant.EnrollConstant;
import com.yizhi.application.domain.Enroll;
import com.yizhi.application.domain.EnrollRecord;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.enroll.application.enums.EvenType;
import com.yizhi.application.mapper.EnrollMapper;
import com.yizhi.application.mapper.EnrollRecordMapper;
import com.yizhi.application.service.EnrollRecordService;
import com.yizhi.application.service.EnrollService;
import com.yizhi.application.util.EnrollEvenSendMessage;
import com.yizhi.enroll.application.vo.AuditVO;
import com.yizhi.enroll.application.vo.EnrollerVo;
import com.yizhi.enroll.application.vo.TrainningEnrollVO;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.constant.ReturnCode;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 记录学员报名信息 服务实现类
 * </p>
 *
 * @author bob123
 * @since 2018-03-07
 */
@Service
public class EnrollRecordServiceImpl extends ServiceImpl<EnrollRecordMapper, EnrollRecord> implements EnrollRecordService {

    private static final Logger LOGGER = LoggerFactory.getLogger(EnrollRecordServiceImpl.class);

    @Autowired
    IdGenerator idGenerator;
    @Resource
    private EnrollService enrollService;
    @Resource
    private EnrollRecordService enrollRecordService;
    @Resource
    private EnrollRecordMapper enrollRecordMapper;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private CloudEventPublisher cloudEventPublisher;
    @Autowired
    private EnrollEvenSendMessage enrollEvenSendMessage;
    @Autowired
    private TrainingProjectClient trainingProjectClient;

    @Autowired
    EnrollMapper enrollMapper;
    
    @Override
    public List<EnrollerVo> exportRegisteredList(Map<String, Object> map) {
        return enrollRecordMapper.exportRegisteredList(map);
    }

    /**
     * 报名管理分页查询
     *
     * @param map       条件
     * @param pageIndex 页码
     * @param pageSize  页数量
     * @return
     */
    @Override
    public Page<EnrollerVo> listEnroll(Map<String, Object> map, Page<EnrollerVo> page) {
        return page.setRecords(enrollRecordMapper.list(map, page));
    }

    @Override
    public Long insertTrEnrollRecord(Long trainingProjectId) {
        Long accountId = 1L;
        Long orgId = 1L;
        Date now = new Date();
        RequestContext requestContext = ContextHolder.get();
        if (null != requestContext) {
            if (null != requestContext.getAccountId()) {
                accountId = requestContext.getAccountId();
            }
            if (null != requestContext.getOrgId()) {
                orgId = requestContext.getOrgId();
            }
        }

        //检查报名人数是否满了
        Enroll enrollConfig = enrollService.selectByProjectId(trainingProjectId);
        if(enrollConfig.getPersonLimitNum() > 0){
        	Integer count = enrollRecordMapper.selectEnrollCountByProjectId(trainingProjectId);
            if(count != null && count >= enrollConfig.getPersonLimitNum()){
            	 return null;
            }
        }
        
        EnrollRecord enrollRecord = new EnrollRecord();
        enrollRecord.setId(idGenerator.generate());
        enrollRecord.setTrainingProjectId(trainingProjectId);
        enrollRecord.setAccountId(accountId);
        enrollRecord.setOrgId(orgId);
        enrollRecord.setApplyTime(now);
        //通过项目ID获取项目下学员是否需要审核 ,0：不需要审核，1：待审核
        Integer auditStatus = enrollService.queryNeedAuditByTrainingProjectId(trainingProjectId);
        enrollRecord.setAuditStatus(auditStatus);
       
        if (enrollConfig != null && enrollConfig.getEnablePay() != null && enrollConfig.getEnablePay() == 1) {
            enrollRecord.setExpenditure(enrollConfig.getActualPrice());
        }

        if (this.insert(enrollRecord)) {
        	/*//2024-11-13 项目不使用MQ进行异步操作，注释代码，添加tp_student_enroll_passed表数据在中台调用项目服务接口
            // 如果不需要审核，需要给培训发送异步消息
            if (auditStatus.equals(0)) {
                Enroll enroll = new Enroll();
                enroll.setCompanyId(requestContext.getCompanyId());
                enroll.setSiteId(requestContext.getSiteId());
                enroll.setTrainingProjectId(trainingProjectId);
                enroll = enroll.selectOne(new EntityWrapper(enroll));

                if (null != enroll) {
                    try {
                        TrainingProjectEvent event = TrainingProjectEvent.getInstanceEnroll(
                                enroll.getId(), accountId, now, trainingProjectId, requestContext.getSiteId());
                        EventWrapper<TrainingProjectEvent> eventWrapper = new EventWrapper<>(enroll.getId(), event);
                        taskExecutor.asynExecute(new AbstractTaskHandler() {
                            @Override
                            public void handle() {
                                try {
                                    cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, eventWrapper);
                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                            }
                        });
                    } catch (Exception e) {
                        LOGGER.error("不需要审核的报名，给培训发送异步消息失败：", e);
                    }
                }
            }*/
        }
        return enrollRecord.getId();
    }

    @Override
    public boolean auditEnroll(AuditVO vo) {
        Date now = new Date();
        EnrollRecord enrollRecord = new EnrollRecord();
        enrollRecord.setTrainingProjectId(vo.getTrainingProjectId());
        enrollRecord.setAccountId(vo.getAccountId());
        // 待审核
        enrollRecord.setAuditStatus(1);
        List<EnrollRecord> records = enrollRecordMapper.selectList(new EntityWrapper<>(enrollRecord));
        if (CollectionUtils.isEmpty(records)){
            return false;
        }
        if (records.size() > 1) {
            for (Integer i = 1; i < records.size(); i++) {
                //之所以更改是因为会出现一个人并发脏数据，所以需要清除只留一条记录
                enrollRecordMapper.deleteById(records.get(i));
            }
        }
        EnrollRecord selectOne = records.get(0);
        if (selectOne == null) {
            return false;
        }
        selectOne.setAuditStatus(vo.getAuditStatus());

        Long accountId = 1L;
        String accountName = "bob";
        RequestContext requestContext = ContextHolder.get();
        if (null != requestContext) {
            if (null != requestContext.getAccountId()) {
                accountId = requestContext.getAccountId();
            }
            if (null != requestContext.getAccountName()) {
                accountName = requestContext.getAccountName();
            }
        }

        selectOne.setAuditUserId(accountId);
        selectOne.setAuditUserName(accountName);
        selectOne.setAuditTime(now);
        boolean result = enrollRecordMapper.updateById(selectOne) > 0;

        Enroll enroll = new Enroll();
        enroll.setCompanyId(requestContext.getCompanyId());
        enroll.setSiteId(requestContext.getSiteId());
        enroll.setTrainingProjectId(vo.getTrainingProjectId());
        enroll = enroll.selectOne(new EntityWrapper(enroll));

        if (result) {
            if (selectOne.getAuditStatus().equals(2)) {

                try {
                    TrainingProjectEvent event = TrainingProjectEvent.getInstanceEnroll(
                            enroll.getId(), selectOne.getAccountId(), now, vo.getTrainingProjectId(), requestContext.getSiteId());
                    EventWrapper<TrainingProjectEvent> eventWrapper = new EventWrapper<>(enroll.getId(), event);
                    taskExecutor.asynExecute(new AbstractTaskHandler() {
                        @Override
                        public void handle() {
                            try {
                                cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, eventWrapper);
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
                } catch (Exception e) {
                    LOGGER.error("需要审核的报名，给培训发送异步消息失败：", e);
                }
            }
        }

        EnrollRecord enrollRecord1 = new EnrollRecord();
        enrollRecord1.setTrainingProjectId(vo.getTrainingProjectId());
        enrollRecord1.setAccountId(vo.getAccountId());
        enrollRecord1 = enrollRecordMapper.selectOne(enrollRecord1);

        if (null == enrollRecord1){
            LOGGER.info("用户："+vo.getAccountId()+"无报名记录！");
            return result;
        }
        if ( enrollRecord1.getAuditStatus() == 2) {
            //审核成功发消息
            try {
                if (enroll != null) {
                    TrainingProjectVo one = trainingProjectClient.getOne(enroll.getTrainingProjectId());
                    if (one != null) {
                        Enroll finalEnroll = enroll;
                        if (vo.getAccountId() != null) {
                            LOGGER.info("accountId:" + vo.getAccountId());
                            taskExecutor.asynExecute(new AbstractTaskHandler() {
                                @Override
                                public void handle() {
                                    enrollEvenSendMessage.evenSendMessage(one, finalEnroll.getId(), finalEnroll.getEndTime(), vo.getAccountId(), EvenType.TRAINING_AUDIT_PASS);
                                }
                            });
                        } else {
                            LOGGER.info("accountId:" + vo.getAccountId() + "为空！！");
                        }
                    }
                }else {
                    LOGGER.info("enroll为空！");
                }
            } catch (Exception e) {
                e.printStackTrace();
                LOGGER.info("报名审核通过发消息异常");
            }
        } else if ( enrollRecord1.getAuditStatus() == 3){
            //审核不通过发消息
            try {
                if (enroll != null) {
                    TrainingProjectVo one = trainingProjectClient.getOne(vo.getTrainingProjectId());
                    if (vo.getAccountId() != null) {
                        LOGGER.info("accountId:" + vo.getAccountId());
                        Enroll finalEnroll1 = enroll;
                        taskExecutor.asynExecute(new AbstractTaskHandler() {
                            @Override
                            public void handle() {
                                enrollEvenSendMessage.evenSendMessage(one, finalEnroll1.getId(), finalEnroll1.getEndTime(), vo.getAccountId(), EvenType.TRAINING_AUDIT_FAIL);
                            }
                        });
                    } else {
                        LOGGER.info("accountId:" + vo.getAccountId() + "为空！！");
                    }
                }else {
                    LOGGER.info("enroll为空！");
                }
            } catch (Exception e) {
                e.printStackTrace();
                LOGGER.info("报名审核通过发消息异常");
            }
        }
        return result;
    }

    @Override
    public List<TrainningEnrollVO> selectTrainningEnroll(List<Long> idList) {
        List<TrainningEnrollVO> list = new ArrayList<TrainningEnrollVO>();
        Long accountId = 0L;
        Long companyId = 0L;
        Long orgId = 0L;
        Long siteId = 0L;

        RequestContext requestContext = ContextHolder.get();
        if (null != requestContext) {
            if (null != requestContext.getAccountId()) {
                accountId = requestContext.getAccountId();
            }
            if (null != requestContext.getCompanyId()) {
                companyId = requestContext.getCompanyId();
            }
            if (null != requestContext.getOrgId()) {
                orgId = requestContext.getOrgId();
            }
            if (null != requestContext.getSiteId()) {
                siteId = requestContext.getSiteId();
            }
        }

        if (idList != null) {
            for (Long trainingProjectId : idList) {
                TrainningEnrollVO vo = new TrainningEnrollVO();
                Map<String, Object> map = new HashMap<String, Object>();
                map.put("training_project_id", trainingProjectId);
                map.put("company_id", companyId);
                map.put("org_id", orgId);
                map.put("site_id", siteId);
                List<Enroll> selectByMap = enrollService.selectByMap(map);
                if (selectByMap.size() == 0) {
                    continue;
                }
                Enroll enroll = selectByMap.get(0);
                vo.setProjectId(trainingProjectId);
                vo.setStartTime(enroll.getStartTime());
                vo.setEnrollId(enroll.getId());
                vo.setEndTime(enroll.getEndTime());
                if (enroll.getNeedAudit() == EnrollConstant.ZERO) {
                    Map<String, Object> map1 = new HashMap<String, Object>();
                    map1.put("training_project_id", trainingProjectId);
                    map1.put("account_id", accountId);
                    List<EnrollRecord> enrollRecord = enrollRecordService.selectByMap(map1);
                    if (enrollRecord.size() > 0) {
                        vo.setIsEnroll(EnrollConstant.FIRST);
                    } else {
                        vo.setIsEnroll(EnrollConstant.ZERO);
                    }
                } else {
                    Map<String, Object> map1 = new HashMap<String, Object>();
                    map1.put("training_project_id", trainingProjectId);
                    map1.put("account_id", accountId);
                    List<EnrollRecord> enrollRecordList = enrollRecordService.selectByMap(map1);
                    if (enrollRecordList.size() > 0) {
                        EnrollRecord enrollRecord = enrollRecordList.get(0);
                        if (enrollRecord.getAuditStatus() == EnrollConstant.TWO) {
                            vo.setIsEnroll(EnrollConstant.FIRST);
                        } else {
                            vo.setIsEnroll(EnrollConstant.ZERO);
                        }
                    } else {
                        vo.setIsEnroll(EnrollConstant.ZERO);
                    }
                }
                list.add(vo);
            }
        }
        return list;
    }

    @Override
    public Integer updateState(EnrollRecord enrollRecord) {
        Long id = enrollRecord.getId();
        Integer auditStatus = enrollRecord.getAuditStatus();
        return enrollRecordMapper.updateState(id, auditStatus);
    }
}
